memvar			.EQU $CC00
x_pos			.EQU memvar
y_pos			.EQU memvar+1
dir			.EQU memvar+2
level			.EQU memvar+3


.org	0

.org $40
  reti
.org $48
  reti
.org $50
  reti
.org $58
  reti
.org $60
  reti


.org $100
  nop
  jp begin

;Nintendo Scrolling Title Graphic
;***********************************************
.byte	0ceh,0edh,66h,66h,0cch,13,0,11,3,73h
.byte	0,83h,0,12,0,13,0,8,17,31,88h,89h,0
.byte	14,0dch,0cch,6eh,0e6h,0ddh,0ddh,0d9h
.byte	99h,0bbh,0bbh,67h,63h,6eh,0eh,0ech,0cch
.byte	0ddh,0dch,99h,09fh,0bbh,0b9h,33h,3eh
;***********************************************

.byte   "Dungeon Escape  "              ;Title of the game
.byte	0,0,0				;Not used
.byte	3 				;Cartridge type: Rom only
					; 0 - Rom only      3 - ROM+MBC1+RAM+Battery
					; 1 - ROM+MBC1      5 - ROM+MBC2
					; 2 - ROM+MBC1+RAM  6 - ROM+MBC2+Battery
.byte	0				;Rom Size:
					; 0 - 256kBit =  32kB =  2 banks
					; 1 - 512kBit =  64kB =  4 banks
					; 2 -   1MBit = 128kB =  8 banks
					; 3 -   2MBit = 256kB = 16 banks
					; 4 -   4MBit = 512kB = 32 banks
.byte   2				;Ram Size:
					; 0 - None
					; 1 -  16kBit =  2kB = 1 bank
					; 2 -  64kBit =  8kB = 1 bank
					; 3 - 256kBit = 32kB = 4 banks
.byte	"Vi","ka"			;Manufacturer code:
.byte	1				;Version Number
.byte	0ah				;Complement check
.word	0				;Checksum


begin:
  di
  ld a,0
  ld hl,$FF42		;Screen Y-cord = 0
  ld (hl),a
  ld hl,$FF43		;Screen X-cord = 0
  ld (hl),a
  ld a,%11010011
  ld ($FF40),a
  ld a,%11100100
  ld hl,$FF48		;OBJ 1 pal
  ld (hl),a
  inc hl		;OBJ 2 pal
  ld (hl),a
  call setrealcolours

;-- Set up variables for new game --
  ld a,1
  ld (level),a			;Start level = 1

;-- Initiate new game --
  ld de,playscreen_tiles
  call loadtiles_126		;Load tileset
  ld de,playscreen_map
  call load_wholemap		;Load screen-map

  call load_newlevel		;Load level-data
  call redraw_dungeon		;Draw dungeon-screen
  call redraw_compass		;Draw the compass

main:
  call readjoypad		;Read joypad to A
  cp $40			;Is it UP?
  call z,go_up
  call readjoypad		;Read joypad to A
  cp $80			;Is it DOWN?
  call z,go_down
  call readjoypad		;Read joypad to A
  cp $11			;Is it right+A?
  call z,go_right
  call readjoypad
  cp $21			;Is it left+A?
  call z,go_left
  call readjoypad
  cp $10			;buton right?
  call z,turn_right
  call readjoypad
  cp $20			;buton left?
  call z,turn_left
  jp main



;***********
;** SUB's **
;***********

go_up:
  ld a,(dir)		;Check direction
  cp 1			;North?
  call z,move_north	;Yes, move north
  ld a,(dir)		;Check direction
  cp 2			;East?
  call z,move_east	;Yes, move east
  ld a,(dir)		;Check direction
  cp 3			;South?
  call z,move_south	;Yes, move south
  ld a,(dir)		;Check direction
  cp 4			;West?
  call z,move_west	;Yes, move west
  call redraw_dungeon
  ret

go_down:
  ld a,(dir)		;Check direction
  cp 1			;North?
  call z,move_south	;Yes, move south
  ld a,(dir)		;Check direction
  cp 2			;East?
  call z,move_west	;Yes, move west
  ld a,(dir)		;Check direction
  cp 3			;South?
  call z,move_north	;Yes, move north
  ld a,(dir)		;Check direction
  cp 4			;West?
  call z,move_east	;Yes, move east
  call redraw_dungeon
  ret

go_right:
  ld a,(dir)		;Check direction
  cp 1			;North?
  call z,move_east	;Yes, move East
  ld a,(dir)		;Check direction
  cp 2			;East?
  call z,move_south	;Yes, move south
  ld a,(dir)		;Check direction
  cp 3			;South?
  call z,move_west	;Yes, move west
  ld a,(dir)		;Check direction
  cp 4			;West?
  call z,move_north	;Yes, move north
  call redraw_dungeon
  ret

go_left:
  ld a,(dir)		;Check direction
  cp 1			;North?
  call z,move_west	;Yes, move west
  ld a,(dir)		;Check direction
  cp 2			;East?
  call z,move_north	;Yes, move north
  ld a,(dir)		;Check direction
  cp 3			;South?
  call z,move_east	;Yes, move east
  ld a,(dir)		;Check direction
  cp 4			;West?
  call z,move_south	;Yes, move south
  call redraw_dungeon
  ret

turn_right:
  ld a,(dir)
  inc a
  ld (dir),a
  cp 5
  call z,resetdir
  call redraw_compass
  call redraw_dungeon
  call wait_padreleased
  ret

turn_left:
  ld a,(dir)
  dec a
  ld (dir),a
  cp 0
  call z,resetdir2
  call redraw_compass
  call redraw_dungeon
  call wait_padreleased
  ret

resetdir:
  ld a,(dir)
  ld a,1
  ld (dir),a
  ret
resetdir2:
  ld a,(dir)
  ld a,4
  ld (dir),a
  ret


move_north:
  ld a,(y_pos)
  dec a
  ld (y_pos),a
  call wait_padreleased
  ret
move_south:
  ld a,(y_pos)
  inc a
  ld (y_pos),a
  call wait_padreleased
  ret
move_east:
  ld a,(x_pos)
  inc a
  ld (x_pos),a
  call wait_padreleased
  ret
move_west:
  ld a,(x_pos)
  dec a
  ld (x_pos),a
  call wait_padreleased
  ret


redraw_compass:
  ld a,(dir)
  ld b,10
  add a,b			;Set right tilenr.
  ld hl,$9800+(32*13)+17
  call wrAtohl
  ret


;**************************************************************
;** Load info from a new level (x-start, y-start, direction) **
;**************************************************************
load_newlevel:
  ld a,(level)
  cp 1
  call z,set_level1
  jp loadleveldata
  ret
set_level1		;Set HL to point at level1
  ld hl,level1
  ret

loadleveldata:		;Load the actual level-data HL=level
  ld c,32		;Width=32
ldat1:
  ld b,32		;Height=32
ldat2:
  ld a,(hl)
  cp 1
  jp z,ldatfin			;player, up
  cp 2
  jp z,ldatfin			;player, right
  cp 3
  jp z,ldatfin			;player, down
  cp 4
  jp z,ldatfin			;player, left
  inc hl
  dec b
  jr nz,ldat2
  dec c
  jr nz,ldat1
ldatfin:
  ld (dir),a			;dir
  ld a,b
  ld a,32
  sbc a,b
  ld (x_pos),a			;x_pos
  ld a,32
  sbc a,c
  ld (y_pos),a			;y_pos
  ret


;***************************
;** Redraw dungeon screen **
;***************************
redraw_dungeon:
  call clear_dungeonscreen	;Clear dungeonscreen
  ld a,(level)			;Check which level
  cp 1				;Is it 1?
  call z,set_level1		;Yes, setup level1

  call clear_dungeonscreen	;Clear dungeonscreen
  ld a,(level)			;Check which level
  cp 1				;Is it 1?
  call z,set_level1		;Yes, setup level1

  ld a,(dir)			;Check hero direction
  cp 1				;Is it 1 (north)
  jp z,draw_dungeon_up		;Yes, draw dungeon north
  cp 2
  jp z,draw_dungeon_right
  cp 3
  jp z,draw_dungeon_down
  cp 4
  jp z,draw_dungeon_left
  ret

;*****************************************************
;** Set up things for drawing when hero faces north **
;*****************************************************
draw_dungeon_up:
  ld a,(x_pos)
setxp:
  inc hl
  dec a
  cp 0
  jp nz,setxp
  dec hl
  dec hl
  dec hl
  ld a,(y_pos)
  dec a
  dec a
  dec a
setyp:
  ld b,32
inc32hl:
  inc hl
  dec b
  jr nz,inc32hl
  dec a
  cp 0
  jp nz,setyp			;HL now points to wall1
drawwalls:
  call draw1
  inc hl
  call draw2
  inc hl
  call draw3
  inc hl
  call draw4
  inc hl
  call draw5
  inc hl
  call draw6
  inc hl
  call draw7
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  call inchl32
  call draw8
  inc hl
  call draw9
  inc hl
  inc hl
  inc hl
  call draw11
  dec hl
  call draw10
  dec hl
  dec hl
  call inchl32
  call draw12
  inc hl
  inc hl
  call draw13
  dec hl
  dec hl
  call inchl32
  call draw14
  inc hl
  inc hl
  call draw15
  dec hl
  call dechl32
  call dechl32
  call draw16
  call inchl32
  call draw17
  ret


draw_dungeon_right:
  ld a,(x_pos)
setxp1:
  inc hl
  dec a
  cp 0
  jp nz,setxp1
  inc hl
  inc hl
  inc hl
  ld a,(y_pos)
  dec a
  dec a
  dec a
setyp1:
  ld b,32
inc32hl1:
  inc hl
  dec b
  jr nz,inc32hl1
  dec a
  cp 0
  jp nz,setyp1			;HL now points to wall1
drawwalls2:
  call draw1
  call inchl32
  call draw2
  call inchl32
  call draw3
  call inchl32
  call draw4
  call inchl32
  call draw5
  call inchl32
  call draw6
  call inchl32
  call draw7
  call dechl32
  call dechl32
  call dechl32
  call dechl32
  call dechl32
  dec hl
  call draw8
  call inchl32
  call draw9
  call inchl32
  call inchl32
  call inchl32
  call draw11
  call dechl32
  call draw10
  dec hl
  call dechl32
  call dechl32
  call draw12
  call inchl32
  call inchl32
  call draw13
  call dechl32
  call dechl32
  dec hl
  call draw14
  call inchl32
  call inchl32
  call draw15
  inc hl
  inc hl
  call dechl32
  call draw16
  dec hl
  call draw17
  ret


draw_dungeon_down:
  ld a,(x_pos)
setxp2:
  inc hl
  dec a
  cp 0
  jp nz,setxp2
  inc hl
  inc hl
  inc hl
  ld a,(y_pos)
  inc a
  inc a
  inc a
setyp2:
  ld b,32
inc32hl2:
  inc hl
  dec b
  jr nz,inc32hl2
  dec a
  cp 0
  jp nz,setyp2			;HL now points to wall1
drawwalls3:
  call draw1
  dec hl
  call draw2
  dec hl
  call draw3
  dec hl
  call draw4
  dec hl
  call draw5
  dec hl
  call draw6
  dec hl
  call draw7
  inc hl
  inc hl
  inc hl
  inc hl
  inc hl
  call dechl32
  call draw8
  dec hl
  call draw9
  dec hl
  dec hl
  dec hl
  call draw11
  inc hl
  call draw10
  inc hl
  inc hl
  call dechl32
  call draw12
  dec hl
  dec hl
  call draw13
  inc hl
  inc hl
  call dechl32
  call draw14
  dec hl
  dec hl
  call draw15
  inc hl
  call inchl32
  call inchl32
  call draw16
  call dechl32
  call draw17
  ret


draw_dungeon_left:
  ld a,(x_pos)
setxp3:
  inc hl
  dec a
  cp 0
  jp nz,setxp3
  dec hl
  dec hl
  dec hl
  ld a,(y_pos)
  inc a
  inc a
  inc a
setyp3:
  ld b,32
inc32hl3:
  inc hl
  dec b
  jr nz,inc32hl3
  dec a
  cp 0
  jp nz,setyp3			;HL now points to wall1
drawwalls4:
  call draw1
  call dechl32
  call draw2
  call dechl32
  call draw3
  call dechl32
  call draw4
  call dechl32
  call draw5
  call dechl32
  call draw6
  call dechl32
  call draw7
  call inchl32
  call inchl32
  call inchl32
  call inchl32
  call inchl32
  inc hl
  call draw8
  call dechl32
  call draw9
  call dechl32
  call dechl32
  call dechl32
  call draw11
  call inchl32
  call draw10
  inc hl
  call inchl32
  call inchl32
  call draw12
  call dechl32
  call dechl32
  call draw13
  call inchl32
  call inchl32
  inc hl
  call draw14
  call dechl32
  call dechl32
  call draw15
  dec hl
  dec hl
  call inchl32
  call draw16
  inc hl
  call draw17
  ret



;*****************************************
;** This draw the walls for the dungeon **
;*****************************************
draw1:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+1
  ld c,4
  call draw_walltype1
  ret
draw2:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+3
  ld c,4
  call draw_walltype1
  ret
draw3:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+5
  ld c,4
  call draw_walltype1
  ret
draw4:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+7
  ld c,4
  call draw_walltype1
  ret
draw5:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+9
  ld c,4
  call draw_walltype1
  ret
draw6:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+11
  ld c,4
  call draw_walltype1
  ret
draw7:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*8)+13
  ld c,4
  call draw_walltype1
  ret
draw8:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*7)+1
  call draw_walltype2_1
  ret
draw9:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*7)+1
  ld c,6
  call draw_walltype1
  ld de,$9800+(32*7)+3
  ld c,6
  call draw_walltype1
  ld de,$9800+(32*7)+5
  call draw_walltype2_1
  ret
draw10:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*7)+13
  ld c,6
  call draw_walltype1
  ld de,$9800+(32*7)+11
  ld c,6
  call draw_walltype1
  ld de,$9800+(32*7)+9
  call draw_walltype2_3
  ret
draw11:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*7)+13
  call draw_walltype2_3
  ret
draw12:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*6)+1
  ld c,8
  call draw_walltype1
  ld de,$9800+(32*6)+3
  call draw_walltype2_2
  ret
draw13:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*6)+13
  ld c,8
  call draw_walltype1
  ld de,$9800+(32*6)+11
  call draw_walltype2_4
  ret
draw14:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*5)+1
  call draw_walltype2_5
  ret
draw15:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*5)+13
  call draw_walltype2_6
  ret
draw16:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*7)+5
  ld c,6
  call draw_walltype1
  ld de,$9800+(32*7)+7
  ld c,6
  call draw_walltype1
  ld de,$9800+(32*7)+9
  ld c,6
  call draw_walltype1
  ret
draw17:
  ld a,(hl)
  cp 5					;Time to draw?
  ret nz				;No, return
  ld de,$9800+(32*6)+3
  ld c,8
  call draw_walltype1
  ld de,$9800+(32*6)+5
  ld c,8
  call draw_walltype1
  ld de,$9800+(32*6)+7
  ld c,8
  call draw_walltype1
  ld de,$9800+(32*6)+9
  ld c,8
  call draw_walltype1
  ld de,$9800+(32*6)+11
  ld c,8
  call draw_walltype1
  ret



draw_walltype1:
  ld a,59			;Tile nr.59
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,draw_walltype1
  ret


draw_walltype2_1:
  ld a,60
  call wrAtode
  inc de
  ld a,61
  call wrAtode
  dec de
  call incde32
  ld c,2
drw1:
  ld a,62
  call wrAtode
  inc de
  call wrAtode
  ld (de),a
  dec de
  call incde32
  dec c
  jr nz,drw1
  ld a,63
  ld c,2
drw2:
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,drw2
  ld a,65
  call wrAtode
  inc de
  dec a
  call wrAtode
  ret

draw_walltype2_2:
  ld a,60
  call wrAtode
  inc de
  ld a,61
  call wrAtode
  dec de
  call incde32
  ld c,3
drw3:
  ld a,62
  call wrAtode
  inc de
  call wrAtode
  ld (de),a
  dec de
  call incde32
  dec c
  jr nz,drw3
  ld a,63
  ld c,3
drw4:
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,drw4
  ld a,65
  call wrAtode
  inc de
  dec a
  call wrAtode
  ret

draw_walltype2_3:
  ld a,67
  call wrAtode
  inc de
  ld a,66
  call wrAtode
  dec de
  call incde32
  ld c,2
drw5:
  ld a,68
  call wrAtode
  inc de
  call wrAtode
  ld (de),a
  dec de
  call incde32
  dec c
  jr nz,drw5
  ld a,69
  ld c,2
drw6:
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,drw6
  ld a,70
  call wrAtode
  inc de
  inc a
  call wrAtode
  ret


draw_walltype2_4:
  ld a,67
  call wrAtode
  inc de
  ld a,66
  call wrAtode
  dec de
  call incde32
  ld c,3
drw7:
  ld a,68
  call wrAtode
  inc de
  call wrAtode
  ld (de),a
  dec de
  call incde32
  dec c
  jr nz,drw7
  ld a,69
  ld c,3
drw8:
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,drw8
  ld a,70
  call wrAtode
  inc de
  inc a
  call wrAtode
  ret

draw_walltype2_5:
  ld a,60
  call wrAtode
  inc de
  ld a,61
  call wrAtode
  dec de
  call incde32
  ld c,4
drw9:
  ld a,62
  call wrAtode
  inc de
  call wrAtode
  ld (de),a
  dec de
  call incde32
  dec c
  jr nz,drw9
  ld a,63
  ld c,4
drw10:
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,drw10
  ld a,65
  call wrAtode
  inc de
  dec a
  call wrAtode
  ret


draw_walltype2_6:
  ld a,67
  call wrAtode
  inc de
  ld a,66
  call wrAtode
  dec de
  call incde32
  ld c,4
drw11:
  ld a,68
  call wrAtode
  inc de
  call wrAtode
  ld (de),a
  dec de
  call incde32
  dec c
  jr nz,drw11
  ld a,69
  ld c,4
drw12:
  call wrAtode
  inc de
  call wrAtode
  dec de
  call incde32
  dec c
  jr nz,drw12
  ld a,70
  call wrAtode
  inc de
  inc a
  call wrAtode
  ret


clear_dungeonscreen:		;Clear dungeonscreen
  ld hl,$9800+(32*5)+1		;Set offset
  ld a,0			;A blank tile
  ld d,10			;Height=10
clrd1:
  ld c,14			;Width=14
clrd2:
  call wrAtohl
  inc hl
  dec c
  jr nz,clrd2
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  dec hl
  call inchl32
  dec d
  jr nz,clrd1
  ret



inchl32:
  ld b,32
doincr:
  inc hl
  dec b
  jr nz,doincr
  ret

dechl32:
  ld b,32
dodecr:
  dec hl
  dec b
  jr nz,dodecr
  ret

incde32:
  ld b,32
doincr2:
  inc de
  dec b
  jr nz,doincr2
  ret

;************
;*** DATA ***
;************
#include "standard.z80"
playscreen_map:
#include "rpg_map.z80"
playscreen_tiles:
#include "rpg_tile.z80"

level1:
#include "level1.map"

.end




